/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Id: DBObserver.java 541508 2007-05-25 01:54:12Z vgritsenko $
 */

package org.apache.xindice.core;

import org.apache.xindice.core.data.Key;
import org.apache.xindice.core.data.Record;
import org.apache.xindice.util.Configuration;

import org.w3c.dom.Document;

import java.util.Map;

/**
 * Observer for Xindice DB activities
 *
 * @version $Revision: 541508 $, $Date: 2007-05-24 21:54:12 -0400 (Thu, 24 May 2007) $
 */
public abstract class DBObserver {

    private static final DBObserver NOOP = new DBObserver() {
        public void setDatabaseConfig(Database db, Map collections, Configuration cfg) {
        }

        public void setCollectionConfig(Collection col, Configuration cfg) {
        }

        public void flushDatabaseConfig(Database db, Configuration cfg) {
        }

        public void dropCollection(Collection col) {
        }

        public void createCollection(Collection col) {
        }

        public void putDocument(Collection col, Key key, Document document, boolean create) {
        }

        public void loadDocument(Collection col, Record record, Document document) {
        }

        public void dropDocument(Collection col, Key key) {
        }
    };

    private static DBObserver instance = NOOP;

    /**
     * Sets the default observer instance
     */
    public static void setInstance(DBObserver obs) {
        instance = (null == obs) ? NOOP : obs;
    }

    /**
     * Returns the observer instance, must be non-null
     */
    public static DBObserver getInstance() {
        return instance;
    }

    /**
     * Called after Database.setConfig()
     */
    public abstract void setDatabaseConfig(Database db, Map collections, Configuration cfg);

    /**
     * Called after Collection.setConfig()
     */
    public abstract void setCollectionConfig(Collection col, Configuration cfg);

    /**
     * Called after Database.flushConfig()
     */
    public abstract void flushDatabaseConfig(Database db, Configuration cfg);

    /**
     * Called before Collection.drop()
     */
    public abstract void dropCollection(Collection col) throws DBException;

    /**
     * Called after Collection.create()
     */
    public abstract void createCollection(Collection col) throws DBException;

    /**
     * Called after Collection.putDocument()
     */
    public abstract void putDocument(Collection col, Key key, Document document, boolean create) throws DBException;

    /**
     * Called after Collection.getDocument()
     */
    public abstract void loadDocument(Collection col, Record record, Document document) throws DBException;

    /**
     * Called before Collection.remove(key)
     */
    public abstract void dropDocument(Collection col, Key key) throws DBException;
}
