/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Id: DocumentCache.java 594581 2007-11-13 16:59:33Z vgritsenko $
 */

package org.apache.xindice.core.cache;

import org.apache.xindice.core.Collection;
import org.apache.xindice.core.data.Entry;
import org.apache.xindice.core.data.Key;
import org.apache.xindice.core.data.Value;

import java.util.Map;

/**
 * DocumentCache implements a simple caching system for
 * Collection resources.
 *
 * @version $Revision: 594581 $, $Date: 2007-11-13 11:59:33 -0500 (Tue, 13 Nov 2007) $
 */
public interface DocumentCache {

    /** Compressed document resource type */
    int COMPRESSED   = 1;

    /** Uncompressed document resource type */
    int UNCOMPRESSED = 2;

    /** Binary resource type */
    int BINARY       = 3;

    /**
     * Cache key consists of collection and resource key
     */
    class CacheKey {
        private final Collection col;
        private final Key key;
        private transient int hashCode;

        public CacheKey(Collection col, Key key) {
            this.col = col;
            this.key = key;
        }

        public Collection getCollection() {
            return col;
        }

        public Key getKey() {
            return key;
        }

        public int hashCode() {
            if (hashCode == 0) {
                hashCode = col.getCanonicalDocumentName(key).hashCode();
            }

            return hashCode;
        }

        public boolean equals(Object o) {
            if (o instanceof DocumentCache.CacheKey) {
                CacheKey k = (CacheKey) o;

                return col == k.col && key.equals(k.key);
            }

            return false;
        }

        public String toString() {
            return col.getCanonicalDocumentName(key);
        }
    }

    /**
     * Obtains resource entry from the cache
     *
     * @param col resource collection
     * @param key resource key
     * @return entry cached resource entry or null if not present
     */
    Entry getEntry(Collection col, Key key);

    /**
     * Obtains resource entry metadata from cache.
     *
     * @param col resource collection
     * @param key resource key
     * @return entry cached resource metadata or null if not present
     */
    Entry getEntryMeta(Collection col, Key key);

    /**
     * Stores resource entry value in the cache
     *
     * @param col resource collection
     * @param key resource key
     * @param type resource type: COMPRESSED, UNCOMPRESSED, BINARY
     * @param value entry value
     * @param meta resource meta attributes map
     */
    void putEntry(Collection col, Key key, int type, Value value, Map meta);

    /**
     * Stores resource meta entry in the cache
     *
     * @param col resource collection
     * @param key resource key
     * @param type resource type: COMPRESSED, UNCOMPRESSED, BINARY
     * @param meta resource metadata attributes map
     */
    void putEntryMeta(Collection col, Key key, int type, Map meta);

    /**
     * Remove resource entry from the cache
     *
     * @param col resource collection
     * @param key resource key
     */
    void removeEntry(Collection col, Key key);
}
