/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Id: IndexMatch.java 564424 2007-08-09 23:40:03Z natalia $
 */

package org.apache.xindice.core.indexer;

import org.apache.xindice.core.data.Key;

/**
 * IndexMatch is used for representing match information.  IndexMatches
 * are the basic unit of information between Indexers and the IndexManager
 * for performing queries and query optimization.
 *
 * @version $Revision: 564424 $, $Date: 2007-08-09 19:40:03 -0400 (Thu, 09 Aug 2007) $
 */
public final class IndexMatch implements Comparable {

    private Key key;    // The Document Key
    private int pos;    // Offset into the stream
    private int len;    // Length of substream
    private short elem; // Element ID (for wildcards)
    private short attr; // Attribute ID (if any)


    public IndexMatch() {
        key = null;
        pos = -1;
        len = -1;
        elem = -1;
        attr = -1;
    }

    public IndexMatch(Key key, int pos, int len, short elem, short attr) {
        this.key = key;
        this.pos = pos;
        this.len = len;
        this.elem = elem;
        this.attr = attr;
    }

    public IndexMatch(Key key, int pos, int len) {
        this.key = key;
        this.pos = pos;
        this.len = len;
        elem = -1;
        attr = -1;
    }

    public IndexMatch(Key key, IndexPattern pattern) {
        this.key = key;
        pos = -1;
        len = -1;
        elem = pattern.getElementID();
        attr = pattern.getAttributeID();
    }

    public IndexMatch(Key key, short elem, short attr) {
        this.key = key;
        pos = -1;
        len = -1;
        this.elem = elem;
        this.attr = attr;
    }

    /**
     * getKey returns the Document Key for the IndexMatch.
     *
     * @return The Key
     */
    public final Key getKey() {
        return key;
    }

    /**
     * getPosition returns the Document position for the Match.
     *
     * @return The Document position
     */
    public final int getPosition() {
        return pos;
    }

    /**
     * getLength returns the Node length for the Match.
     *
     * @return The Node length
     */
    public final int getLength() {
        return len;
    }

    /**
     * getElement returns the Element symbol ID for the Match.
     *
     * @return The Element Symbol ID
     */
    public final short getElement() {
        return elem;
    }

    /**
     * getAttribute returns the Attribute symbol ID for the Match.
     *
     * @return The Attribute Symbol ID
     */
    public final short getAttribute() {
        return attr;
    }

    public int compareTo(Object obj) {
        return key.compareTo(obj);
    }

    public String toString() {
        return key.toString();
    }
}

