/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Id: ResourceTypeReader.java 541508 2007-05-25 01:54:12Z vgritsenko $
 */

package org.apache.xindice.core.meta.inline;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xindice.core.FaultCodes;
import org.apache.xindice.core.data.Value;

/**
 *
 * @version $Revision: 541508 $, $Date: 2007-05-24 21:54:12 -0400 (Thu, 24 May 2007) $
 */
public class ResourceTypeReader implements InlineMetaReader {

    private static final Log log = LogFactory.getLog(ResourceTypeReader.class);

    public static final Integer XML    = new Integer(1);
    public static final Integer BINARY = new Integer(2);


    /**
     * @see org.apache.xindice.core.meta.inline.InlineMetaReader#getVersion()
     */
    public int getVersion() {
        return 1;
    }

    /**
     * @see org.apache.xindice.core.meta.inline.InlineMetaReader#read(Value)
     */
    public InlineMetaMap read(Value data) throws InlineMetaException {
        if (log.isDebugEnabled()) {
            log.debug("ResourceTypeReader.read: data length=" + data.getLength());
        }

        if (data.getLength() != 1) {
            throw new InlineMetaException(FaultCodes.COL_DOCUMENT_MALFORMED,
                                          "Expecting header length of 1");
        }

        Integer type;
        type = new Integer(data.byteAt(0));

        if (!XML.equals(type) && !BINARY.equals(type)) {
            throw new InlineMetaException(FaultCodes.COL_DOCUMENT_MALFORMED,
                                          "Unexpected type value: " + type);
        }

        ResourceTypeMap resourceTypeMap = new ResourceTypeMap();
        resourceTypeMap.put("type", type);
        return resourceTypeMap;
    }

    /**
     * A simple and efficient Map implementation that holds
     * the attributes available from a ResourceType header.
     */
    public static class ResourceTypeMap implements InlineMetaMap {

        private static String[] keys = new String[]{"type"};

        private Integer type;

        /**
         * @see org.apache.xindice.core.meta.inline.InlineMetaMap#containsKey(String)
         */
        public boolean containsKey(String key) {
            return "type".equals(key);
        }

        /**
         * @see org.apache.xindice.core.meta.inline.InlineMetaMap#get(String)
         */
        public Object get(String key) throws InlineMetaException {
            if ("type".equals(key)) {
                return type;
            } else {
                throw new InlineMetaException(FaultCodes.GEN_CRITICAL_ERROR,
                                              "ResourceTypeMap does not accept key '" + key + "'");
            }
        }

        /**
         * @see org.apache.xindice.core.meta.inline.InlineMetaMap#keys()
         */
        public String[] keys() {
            return keys;
        }

        /**
         * @see org.apache.xindice.core.meta.inline.InlineMetaMap#put(String, Object)
         */
        public void put(String key, Object value) throws InlineMetaException {
            if ("type".equals(key)) {
                if (value instanceof Integer) {
                    type = (Integer) value;
                } else {
                    throw new InlineMetaException(FaultCodes.GEN_CRITICAL_ERROR,
                                                  "ResourceTypeMap key 'type' requires an Integer value");
                }
            } else {
                throw new InlineMetaException(FaultCodes.GEN_CRITICAL_ERROR,
                                              "ResourceTypeMap does not accept key '" + key + "'");
            }
        }
    }
}
