/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Id: ByteBuffer.java 541508 2007-05-25 01:54:12Z vgritsenko $
 */

package org.apache.xindice.util;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.Serializable;
import java.io.UnsupportedEncodingException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * ByteBuffer manages volatile arrays of bytes. It is implemented with
 * nearly the same method prototypes as StringBuffer except for the
 * setByteAt and byteAt methods which replace
 * StringBuffer's setCharAt and charAt methods.
 * ByteBuffer also extends the OutputStream abstract class and
 * functions like the ByteArrayOutputStream class.
 * getInputStream returns an InputStream instance for
 * streaming in the content of the ByteBuffer. Because none of the
 * methods in ByteBuffer are synchronized, it is the user's job to
 * properly synchronize calls to any of the methods of this class.
 *
 * @version $Revision: 541508 $, $Date: 2007-05-24 21:54:12 -0400 (Thu, 24 May 2007) $
 */
public final class ByteBuffer extends OutputStream implements Serializable {

    private static final Log log = LogFactory.getLog(ByteBuffer.class);

    static final long serialVersionUID = -3900903004630456844L;

    private transient int length = 0;

    private byte[] buffer = null;
    private boolean locked = false;

    public ByteBuffer(int size) {
        buffer = new byte[size];
    }

    public ByteBuffer(String value) {

        try {

            buffer = value.getBytes("utf-8");
            length = buffer.length;
        } catch (UnsupportedEncodingException e) {

            throw new IllegalStateException("Java doesn't seem to support UTF-8");
        }
    }

    public ByteBuffer(StringBuffer value) {
        this(value.toString());
    }

    public ByteBuffer(byte[] value, boolean copy) {
        length = value.length;
        if (copy) {
            buffer = new byte[length];
            System.arraycopy(value, 0, buffer, 0, length);
        } else {
            buffer = value;
        }
    }

    public ByteBuffer(byte[] value) {
        length = value.length;
        buffer = new byte[length];
        System.arraycopy(value, 0, buffer, 0, length);
    }

    public ByteBuffer() {
        this(64);
    }

    public int capacity() {
        return buffer.length;
    }

    public void ensureCapacity(int size) {
        if (locked) {
            return;
        }
        checkCapacity(size);
    }

    private void checkCapacity(int size) {
        if (size > buffer.length) {
            int newsize = (buffer.length + 1) * 2;
            if (size > newsize) {
                newsize = size;
            }
            byte[] old = buffer;
            buffer = new byte[newsize];
            System.arraycopy(old, 0, buffer, 0, old.length);
        }
    }

    public void pack() {
        if (locked) {
            return;
        }
        if (buffer.length > length) {
            byte[] value = new byte[length];
            System.arraycopy(buffer, 0, value, 0, length);
            buffer = value;
        }
    }

    /* StringBuffer style methods */

    public void append(byte[] value) {
        if (locked) {
            return;
        }
        checkCapacity(length + value.length);
        System.arraycopy(value, 0, buffer, length, value.length);
        length += value.length;
    }

    public void append(char[] value) {

        append(new String(value));
    }

    public void append(String value) {

        try {

            append(value.getBytes("utf-8"));
        } catch (UnsupportedEncodingException e) {

            throw new IllegalStateException("Java doesn't seem to know UTF-8...");
        }
    }

    public void append(StringBuffer value) {
        append(value.toString());
    }

    public void append(ByteBuffer value) {
        try {
            value.writeTo(this);
        } catch (Exception e) {
            if (log.isWarnEnabled()) {
                log.warn("ignored exception", e);
            }
        }
    }

    public void append(boolean value) {
        append(String.valueOf(value));
    }

    public void append(char value) {
        if (locked) {
            return;
        }
        append(new char[]{value});
    }

    public void append(byte value) {
        if (locked) {
            return;
        }
        checkCapacity(length + 1);
        buffer[length++] = value;
    }

    public void append(double value) {
        append(String.valueOf(value));
    }

    public void append(float value) {
        append(String.valueOf(value));
    }

    public void append(int value) {
        append(String.valueOf(value));
    }

    public void append(long value) {
        append(String.valueOf(value));
    }

    public void append(Object value) {
        append(String.valueOf(value));
    }

    public void insert(byte[] value, int pos) {
        if (locked) {
            return;
        }
        if (pos < 0 || pos >= length) {
            throw new IndexOutOfBoundsException();
        }
        checkCapacity(length + value.length);
        byte[] temp = new byte[length - pos];
        System.arraycopy(buffer, pos, temp, 0, temp.length);
        System.arraycopy(temp, 0, buffer, pos + value.length, temp.length);
        System.arraycopy(value, 0, buffer, pos, value.length);
        length += value.length;
    }

    public void insert(char[] value, int pos) {
        if (locked) {
            return;
        }

        insert(new String(value), pos);
    }

    public void insert(String value, int pos) {
        try {

            insert(value.getBytes("utf-8"), pos);
        } catch (UnsupportedEncodingException e) {

            throw new IllegalStateException("Java doesn't seem to support UTF-8");
        }
    }

    public void insert(StringBuffer value, int pos) {

        insert(value.toString(), pos);
    }

    public void insert(ByteBuffer value, int pos) {
        insert(value.toByteArray(), pos);
    }

    public void insert(boolean value, int pos) {
        insert(String.valueOf(value), pos);
    }

    public void insert(char value, int pos) {

        insert(new char[]{value}, pos);
    }

    public void insert(byte value, int pos) {
        if (locked) {
            return;
        }
        if (pos < 0 || pos >= length) {
            throw new IndexOutOfBoundsException();
        }
        checkCapacity(length + 1);
        byte[] temp = new byte[length - pos];
        System.arraycopy(buffer, pos, temp, 0, temp.length);
        System.arraycopy(temp, 0, buffer, pos + 1, temp.length);
        buffer[pos] = value;
        length++;
    }

    public void insert(double value, int pos) {
        insert(String.valueOf(value), pos);
    }

    public void insert(float value, int pos) {
        insert(String.valueOf(value), pos);
    }

    public void insert(int value, int pos) {
        insert(String.valueOf(value), pos);
    }

    public void insert(long value, int pos) {
        insert(String.valueOf(value), pos);
    }

    public void insert(Object value, int pos) {
        insert(String.valueOf(value), pos);
    }

    public void delete(int pos, int width) {
        if (locked) {
            return;
        }
        if (pos + width > length) {
            width = (length - pos);
        }
        byte[] temp = new byte[length - (pos + width)];
        System.arraycopy(buffer, pos + width, temp, 0, temp.length);
        System.arraycopy(temp, 0, buffer, pos, temp.length);
        length -= width;
    }

    public void replace(byte[] value, int pos, int width) {
        delete(pos, width);
        insert(value, pos);
    }

    public void replace(String value, int pos, int width) {
        try {

            replace(value.getBytes("utf-8"), pos, width);
        } catch (UnsupportedEncodingException e) {

            throw new IllegalStateException("Java doesn't seem to know UTF-8");
        }
    }

    public void replace(StringBuffer value, int pos, int width) {

        replace(value.toString(), pos, width);
    }

    public void replace(ByteBuffer value, int pos, int width) {

        replace(value.toByteArray(), pos, width);
    }

    public void replace(boolean value, int pos, int width) {

        replace(String.valueOf(value), pos, width);
    }

    public void replace(char value, int pos, int width) {

        replace(String.valueOf(value), pos, width);
    }

    public void replace(double value, int pos, int width) {

        replace(String.valueOf(value), pos, width);
    }

    public void replace(float value, int pos, int width) {

        replace(String.valueOf(value), pos, width);
    }

    public void replace(int value, int pos, int width) {

        replace(String.valueOf(value), pos, width);
    }

    public void replace(long value, int pos, int width) {

        replace(String.valueOf(value), pos, width);
    }

    public void replace(Object value, int pos, int width) {

        replace(String.valueOf(value), pos, width);
    }

    public byte[] toByteArray() {

        byte[] value = new byte[length];
        System.arraycopy(buffer, 0, value, 0, length);
        return value;
    }

    public String toString() {

        try {

            return new String(buffer, "utf-8");
        } catch (UnsupportedEncodingException e) {

            throw new IllegalStateException("Java doesn't seem to support UTF-8");
        }
    }

    public int length() {
        return length;
    }

    public void setLength(int value) {
        if (locked) {
            return;
        }
        if (value < 0 || value > length) {
            throw new IndexOutOfBoundsException();
        }
        length = value;
    }

    /* ByteArrayOutputStream style methods */

    public int size() {
        return length;
    }

    public void reset() {
        if (locked) {
            return;
        }
        length = 0;
    }

    public byte byteAt(int index) {
        if (index < 0 || index >= length) {
            throw new IndexOutOfBoundsException();
        }
        return buffer[index];
    }

    public void setByteAt(int index, byte value) {
        if (locked) {
            return;
        }
        if (index < 0 || index >= length) {
            throw new IndexOutOfBoundsException();
        }
        buffer[index] = value;
    }

    public void reverse() {
        if (locked) {
            return;
        }
        byte b = 0;
        for (int i = 0; i < length / 2; i++) {
            b = buffer[i];
            buffer[i] = buffer[(length - 1) - i];
            buffer[(length - 1) - i] = b;
        }
    }

    public InputStream getInputStream() {
        return new ByteArrayInputStream(buffer, 0, length);
    }

    public void writeTo(OutputStream out) throws IOException {
        out.write(buffer, 0, length);
    }

    public void chunkTo(OutputStream out, int chunkSize) throws IOException {
        int pos = 0;
        int size = 0;
        while (pos < length) {
            size = length - pos > chunkSize ? chunkSize
                    : length - pos;
            out.write(buffer, pos, size);
            pos += size;
            Thread.yield();
        }
    }

    public void chunkTo(OutputStream out) throws IOException {
        chunkTo(out, 2048);
    }

    public void readFrom(InputStream in) throws IOException {
        if (locked) {
            throw new IOException("ByteBuffer Is Locked");
        }
        int size = in.available();
        checkCapacity(length + size);
        in.read(buffer, length, size);
        length += size;
    }

    public void write(int b) throws IOException {
        if (locked) {
            throw new IOException("ByteBuffer Is Locked");
        }
        checkCapacity(length + 1);
        buffer[length++] = (byte) b;
    }

    public void write(byte b[]) throws IOException {
        if (locked) {
            throw new IOException("ByteBuffer Is Locked");
        }
        checkCapacity(length + b.length);
        System.arraycopy(b, 0, buffer, length, b.length);
        length += b.length;
    }

    public void write(byte b[], int off, int len) throws IOException {
        if (locked) {
            throw new IOException("ByteBuffer Is Locked");
        }
        checkCapacity(length + len);
        System.arraycopy(b, off, buffer, length, len);
        length += len;
    }

    /* Serialization-Related Methods */

    private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
        // Read the array in and set the length correctly
        in.defaultReadObject();
        length = buffer.length;
    }

    private void writeObject(ObjectOutputStream out) throws IOException {
        // Pack the array, so we're not saving crap
        pack();
        out.defaultWriteObject();
    }

    /* Buffer locking related methods */

    public void lock() {
        if (!locked) {
            pack();
            locked = true;
        }
    }

    public boolean isLocked() {
        return locked;
    }
}

