/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Id: HtmlViewerComponent.java 541515 2007-05-25 02:45:06Z vgritsenko $
 */

package org.apache.xindice.webadmin.viewer;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;

import org.apache.xindice.server.Xindice;
import org.apache.xindice.webadmin.WebAdminManager;

/**
 * Xindice Html Viewer.
 *
 * @author <a href="mailto:jmetzner@apache.org">Jan Metzner</a>
 * @version $Revision: 541515 $, $Date: 2007-05-24 22:45:06 -0400 (Thu, 24 May 2007) $
 */
public abstract class HtmlViewerComponent {

    static final Object[][] entitiesArray =
            {{"quot", new Integer(34)}, // " - double-quote
             {"amp", new Integer(38)}, // & - ampersand
             {"lt", new Integer(60)}, // < - less-than
             {"gt", new Integer(62)}, // > - greater-than
             {"nbsp", new Integer(160)}, // non-breaking space
             {"copy", new Integer(169)}, // ? - copyright
             {"reg", new Integer(174)}, // ? - registered trademark
             {"Agrave", new Integer(192)}, // ? - uppercase A, grave accent
             {"Aacute", new Integer(193)}, // ? - uppercase A, acute accent
             {"Acirc", new Integer(194)}, // ? - uppercase A, circumflex accent
             {"Atilde", new Integer(195)}, // ? - uppercase A, tilde
             {"Auml", new Integer(196)}, // ? - uppercase A, umlaut
             {"Aring", new Integer(197)}, // ? - uppercase A, ring
             {"AElig", new Integer(198)}, // ? - uppercase AE
             {"Ccedil", new Integer(199)}, // ? - uppercase C, cedilla
             {"Egrave", new Integer(200)}, // ? - uppercase E, grave accent
             {"Eacute", new Integer(201)}, // ? - uppercase E, acute accent
             {"Ecirc", new Integer(202)}, // ? - uppercase E, circumflex accent
             {"Euml", new Integer(203)}, // ? - uppercase E, umlaut
             {"Igrave", new Integer(204)}, // ? - uppercase I, grave accent
             {"Iacute", new Integer(205)}, // ? - uppercase I, acute accent
             {"Icirc", new Integer(206)}, // ? - uppercase I, circumflex accent
             {"Iuml", new Integer(207)}, // ? - uppercase I, umlaut
             {"ETH", new Integer(208)}, // ? - uppercase Eth, Icelandic
             {"Ntilde", new Integer(209)}, // ? - uppercase N, tilde
             {"Ograve", new Integer(210)}, // ? - uppercase O, grave accent
             {"Oacute", new Integer(211)}, // ? - uppercase O, acute accent
             {"Ocirc", new Integer(212)}, // ? - uppercase O, circumflex accent
             {"Otilde", new Integer(213)}, // ? - uppercase O, tilde
             {"Ouml", new Integer(214)}, // ? - uppercase O, umlaut
             {"Oslash", new Integer(216)}, // ? - uppercase O, slash
             {"Ugrave", new Integer(217)}, // ? - uppercase U, grave accent
             {"Uacute", new Integer(218)}, // ? - uppercase U, acute accent
             {"Ucirc", new Integer(219)}, // ? - uppercase U, circumflex accent
             {"Uuml", new Integer(220)}, // ? - uppercase U, umlaut
             {"Yacute", new Integer(221)}, // ? - uppercase Y, acute accent
             {"THORN", new Integer(222)}, // ? - uppercase THORN, Icelandic
             {"szlig", new Integer(223)}, // ? - lowercase sharps, German
             {"agrave", new Integer(224)}, // ? - lowercase a, grave accent
             {"aacute", new Integer(225)}, // ? - lowercase a, acute accent
             {"acirc", new Integer(226)}, // ? - lowercase a, circumflex accent
             {"atilde", new Integer(227)}, // ? - lowercase a, tilde
             {"auml", new Integer(228)}, // ? - lowercase a, umlaut
             {"aring", new Integer(229)}, // ? - lowercase a, ring
             {"aelig", new Integer(230)}, // ? - lowercase ae
             {"ccedil", new Integer(231)}, // ? - lowercase c, cedilla
             {"egrave", new Integer(232)}, // ? - lowercase e, grave accent
             {"eacute", new Integer(233)}, // ? - lowercase e, acute accent
             {"ecirc", new Integer(234)}, // ? - lowercase e, circumflex accent
             {"euml", new Integer(235)}, // ? - lowercase e, umlaut
             {"igrave", new Integer(236)}, // ? - lowercase i, grave accent
             {"iacute", new Integer(237)}, // ? - lowercase i, acute accent
             {"icirc", new Integer(238)}, // ? - lowercase i, circumflex accent
             {"iuml", new Integer(239)}, // ? - lowercase i, umlaut
             {"igrave", new Integer(236)}, // ? - lowercase i, grave accent
             {"iacute", new Integer(237)}, // ? - lowercase i, acute accent
             {"icirc", new Integer(238)}, // ? - lowercase i, circumflex accent
             {"iuml", new Integer(239)}, // ? - lowercase i, umlaut
             {"eth", new Integer(240)}, // ? - lowercase eth, Icelandic
             {"ntilde", new Integer(241)}, // ? - lowercase n, tilde
             {"ograve", new Integer(242)}, // ? - lowercase o, grave accent
             {"oacute", new Integer(243)}, // ? - lowercase o, acute accent
             {"ocirc", new Integer(244)}, // ? - lowercase o, circumflex accent
             {"otilde", new Integer(245)}, // ? - lowercase o, tilde
             {"ouml", new Integer(246)}, // ? - lowercase o, umlaut
             {"oslash", new Integer(248)}, // ? - lowercase o, slash
             {"ugrave", new Integer(249)}, // ? - lowercase u, grave accent
             {"uacute", new Integer(250)}, // ? - lowercase u, acute accent
             {"ucirc", new Integer(251)}, // ? - lowercase u, circumflex accent
             {"uuml", new Integer(252)}, // ? - lowercase u, umlaut
             {"yacute", new Integer(253)}, // ? - lowercase y, acute accent
             {"thorn", new Integer(254)}, // ? - lowercase thorn, Icelandic
             {"yuml", new Integer(255)}, // ? - lowercase y, umlaut
             {"euro", new Integer(8364)}, // Euro symbol
            };
    static Map entities = new HashMap();

    static {
        for (int i = 0; i < entitiesArray.length; ++i) {
            entities.put(entitiesArray[i][1], entitiesArray[i][0]);
        }
    }

    /**
     * Turns funky characters into HTML entity equivalents
     *
     * @param unescaped unescaped String.
     * @return escaped String.
     */
    public static String htmlescape(String unescaped) {
        return htmlescape(unescaped, false);
    }

    /**
     * Turns funky characters into HTML entity equivalents
     *
     * @param unescaped unescaped String.
     * @param brake     if true a new line will be inserted after every closing tag.
     * @return escaped String.
     */
    public static String htmlescape(String unescaped, boolean brake) {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < unescaped.length(); ++i) {
            char ch = unescaped.charAt(i);
            String entity = (String) entities.get(new Integer((int) ch));
            if (entity == null) {
                if (((int) ch) > 128) {
                    buf.append("&#").append ((int) ch).append(";");
                } else {
                    buf.append(ch);
                }
            } else {
                buf.append("&").append(entity).append(";");
            }
            if (brake && ch == '>') {
                buf.append("\n");
            }
        }
        return buf.toString();
    }

    protected abstract String getName();

    protected ServletOutputStream startViewer(String title, String path, HttpServletResponse res)
            throws ServletException, IOException {

        res.setContentType("text/html; charset=utf-8");
        ServletOutputStream output = res.getOutputStream();
        printHeader(output, title);
        printTabs(output, path);
        return output;
    }

    protected void finishViewer(ServletOutputStream output) throws IOException {
        output.print(FOOTER);
        output.print(HTML_TAIL_SECTION);
        output.flush();
    }

    protected void printHeader(ServletOutputStream output, String title) throws IOException {
        output.print(HTML_HEADER_SECTION);
        output.print(title);
        output.print(END_HTML_HEADER_SECTION);
        output.print(LOGO_SECTION);
    }

    protected void printTabs(ServletOutputStream output, String path) throws IOException {

        output.print(START_TABS_SECTION);

        String[] viewers = new String[]{};
        if (this instanceof HtmlCollectionViewer) {
            viewers = WebAdminManager.getCollectionViewers();
        } else if (this instanceof HtmlResourceViewer) {
            viewers = WebAdminManager.getResourceViewers();
        } else if (this instanceof HtmlDatabaseViewer) {
            viewers = WebAdminManager.getDatabaseViewers();
        }

        for (int i = 0; i < viewers.length; i++) {
            if (viewers[i].equals(getName())) {
                if (i > 0) {
                    output.print(TAB_SPACE_SELECTED);
                }
                output.print(START_TAB_SELECTED);
                output.print(getName());
                output.print(END_TAB);
            } else {
                if (i > 0) {
                    output.print(TAB_SPACE);
                }
                output.print(START_TAB);
                if (viewers[i].length() > 0) {
                    output.print("<a href=\"?viewer="); // TODO do in START_TAB
                    output.print(viewers[i]);
                    output.print("\" class=\"unselected\">");
                    output.print(viewers[i]);
                    output.print("</a>");
                } else {
                    output.print("&nbsp;");
                }
                output.print(END_TAB);
            }
        }
        output.print(END_TABS_SECTION);

        // path section
        output.print(LEFT_PATH_SECTION);
        if (path != null && path.length() > 1) {
            output.print(path);
        } else {
            output.print("/");
        }
        output.print(RIGHT_PATH_SECTION);
        output.print("&nbsp;");
        output.print(END_PATH_SECTION);
    }

    protected void printStartTable(String title, ServletOutputStream output) throws IOException {
        String[] titles = {title};
        printStartTable(titles, output);
    }

    protected void printStartTable(String[] titles, ServletOutputStream output) throws IOException {
        output.print(START_TABLE);
        output.print(START_TABLE_ROW);

        for (int i = 0; i < titles.length; i++) {
            output.print(START_TABLE_BOX_TITLE);
            if (titles[i].length() > 0) {
                output.print(titles[i]);
            } else {
                output.print("&nbsp;");
            }
            output.print(END_TABLE_BOX);
        }

        output.print(END_TABLE_ROW);
    }

    protected void printEndTable(ServletOutputStream output) throws IOException {
        output.print(END_TABLE);
    }

    protected void printTableRow(String content, ServletOutputStream output) throws IOException {
        String[] contents = {content};
        printTableRow(contents, output);
    }

    protected void printTableRow(String[] contents, ServletOutputStream output) throws IOException {
        output.print(START_TABLE_ROW);

        for (int i = 0; i < contents.length; i++) {
            output.print(START_TABLE_BOX);
            if (contents[i] != null && contents[i].length() > 0) {
                output.print(contents[i]);
            } else {
                output.print("&nbsp;");
            }
            output.print(END_TABLE_BOX);
        }

        output.print(END_TABLE_ROW);
    }

    protected void printStartSingleTableBox(ServletOutputStream output) throws IOException {
        output.print(START_TABLE_ROW);
        output.print(START_TABLE_BOX);
    }

    protected void printEndSingleTableBox(ServletOutputStream output) throws IOException {
        output.print(END_TABLE_BOX);
        output.print(END_TABLE_ROW);
    }

    protected void printSearchFromRow(String viewer, String query, String[] ns, ServletOutputStream output)
            throws IOException {

        query = query.replaceAll("\"", "&quot;");

        output.print(START_TABLE_ROW);
        output.print(START_TABLE_BOX);
        output.print(START_SEARCH_FORM);
        output.print(SEARCH_FROM_START_HIDDEN_VIEWER);
        output.print(viewer);
        output.print(SEARCH_FROM_END_HIDDEN_VIEWER);
        output.print(SEARCH_FROM_START_QUERY);
        output.print(query);
        output.print(SEARCH_FROM_END_QUERY);
        output.print(SEARCH_FROM_START_NS_HEADER);

        int nsSize = ns.length / 2;
        for (int i = 1; i < 10; i++) {
            if (i == nsSize) {
                output.print(SEARCH_FROM_START_NS_HEADER_OPT_SEL);
            } else {
                output.print(SEARCH_FROM_START_NS_HEADER_OPT);
            }
            output.print(i);
            output.print(SEARCH_FROM_END_NS_HEADER_OPT);
        }

        output.print(SEARCH_FROM_END_NS_HEADER);

        int thisNs = 1;
        for (int i = 0; i < ns.length; i++) {
            output.print(SEARCH_FROM_START_NS);
            output.print(thisNs);
            output.print(SEARCH_FROM_START_NS_PREFIX);
            output.print(thisNs);
            output.print(SEARCH_FROM_END_NS_PREFIX);
            output.print(ns[i]);
            i++;
            output.print(SEARCH_FROM_START_NS_URI);
            output.print(thisNs);
            output.print(SEARCH_FROM_MIDDLE_NS_URI);
            output.print(ns[i]);
            output.print(SEARCH_FROM_END_NS_URI);
            thisNs++;
        }
        output.print(END_SEARCH_FORM);
        output.print(END_TABLE_BOX);
        output.print(END_TABLE_ROW);
    }

    // ---------------------------------------------------- Protected Constants

    protected static final String HTML_HEADER_SECTION =
            "<html> \n"
            + "<head> \n"
            + "<style type=\"text/css\"><!-- \n"
            + "small { \n"
            + "  font-family: verdana,sans-serif; \n"
            + "  font-size: 8px; \n"
            + "} \n"
            + ".logo { \n"
            + "  font-family: verdana,sans-serif; \n"
            + "  text-align: center; \n"
            + "  font-weight: bold; \n"
            + "  font-size: 50px; \n"
            + "  color: #294563; \n"
            + "  margin-right: 5px; \n"
            + "} \n"
            + ".smallLogo { \n"
            + "  font-family: verdana,sans-serif; \n"
            + "  text-align: center; \n"
            + "  font-weight: normal; \n"
            + "  font-size: 12px; \n"
            + "  color: #294563; \n"
            + "  margin-top: -7px; \n"
            + "} \n"
            + ".tabs { \n"
            + "  background: #f0f0ff; \n"
            + "  color: #294563; \n"
            + "  border-width: 0px; \n"
            + "  padding: 0px 0px 0px 0px; \n"
            + "  margin: 0px 0px 0px 0px; \n"
            + "} \n"
            + ".unselected { \n"
            + "  width: 70px; \n"
            + "  background: #f0f0ff; \n"
            + "  color: #294563; \n"
            + "  font-family: verdana,sans-serif; \n"
            + "  text-align: center; \n"
            + "  font-size: 12px; \n"
            + "} \n"
            + ".selected { \n"
            + "  width: 70px; \n"
            + "  background: #294563; \n"
            + "  color: #f0f0ff; \n"
            + "  font-family: verdana,sans-serif; \n"
            + "  font-weight: bold; \n"
            + "  text-align: center; \n"
            + "  font-size: 12px; \n"
            + "} \n"
            + ".misc { \n"
            + "  background: #294563; \n"
            + "  color: #f0f0ff; \n"
            + "  font-family: verdana,sans-serif; \n"
            + "  font-weight: bold; \n"
            + "  text-align: left; \n"
            + "  font-size: 12px; \n"
            + "} \n"
            + ".misc-right { \n"
            + "  background: #294563; \n"
            + "  color: #f0f0ff; \n"
            + "  font-family: verdana,sans-serif; \n"
            + "  font-weight: bold; \n"
            + "  text-align: right; \n"
            + "  font-size: 12px; \n"
            + "} \n"
            + ".contents { \n"
            + "  width: 100%; \n"
            + "  border-width: 1px; \n"
            + "  border-style: solid; \n"
            + "  border-color: #294563; \n"
            + "} \n"
            + ".contents td { \n"
            + "  border-width: 1px; \n"
            + "  border-style: solid; \n"
            + "  border-color: #294563; \n"
            + "  font-family: verdana,sans-serif; \n"
            + "  text-align: left; \n"
            + "  font-size: 12px; \n"
            + "} \n"
            + ".title { \n"
            + "  background: #4C6C8F; \n"
            + "  color: #f0f0ff; \n"
            + "  font-family: verdana,sans-serif; \n"
            + "  font-weight: bold; \n"
            + "  text-align: left; \n"
            + "  font-size: 12px; \n"
            + "} \n"
            +
            /* xml highlighting */
            ".xml-misc, .xml-element-name, .xml-attribute-name, .xml-attribute-content, "
            + ".xml-element-content, .xml-comment, .xml-processing, .xml-cdata { \n"
            + "  font-family: verdana,sans-serif; \n"
            + "  font-size: 12px; \n"
            + "} \n"
            + ".xml-misc { \n"
            + "  color: #000000; \n"
            + "} \n"
            + ".xml-element-name { \n"
            + "  color: #cc0000; \n"
            + "} \n"
            + ".xml-attribute-name { \n"
            + "  color: #660099; \n"
            + "} \n"
            + ".xml-attribute-content { \n"
            + "  color: #000099; \n"
            + "} \n"
            + ".xml-element-content { \n"
            + "  color: #000000; \n"
            + "} \n"
            + ".xml-comment { \n"
            + "  color: #006600; \n"
            + "} \n"
            + ".xml-processing { \n"
            + "  color: #cc0000; \n"
            + "} \n"
            + ".xml-cdata { \n"
            + "  color: #006600; \n"
            + "} \n"
            +
            /* forms */
            ".search-form, .create-form, .upload-form input { \n"
            + "  color: #294563; \n"
            + "  font-family: verdana,sans-serif; \n"
            + "  font-weight: bold; \n"
            + "  text-align: left; \n"
            + "  font-size: 12px; \n"
            + "  border: 0px none #ffffff; \n"
            + "} \n"
            + ".search-form td[colspan=3] { \n"
            + "  width: 480px; \n"
            + "} \n"
            + ".search-form input { \n"
            + "  border: 2px solid #294563; \n"
            + "  background-color: #f0f0ff; \n"
            + "  width: 200px; \n"
            + "} \n"
            + ".search-form input[name=query] { \n"
            + "  border: 2px solid #294563; \n"
            + "  background-color: #f0f0ff; \n"
            + "  width: 400px; \n"
            + "} \n"
            + ".search-form td, .create-form td, .upload-form td { \n"
            + "  border: 0px none #ffffff; \n"
            + "} \n"
            + ".search-form select { \n"
            + "  font-size: 10px; \n"
            + " background-color: #f0f0ff; \n"
            + "}	 \n"
            + ".create-form td[colspan=2] { \n"
            + "  width: 450px; \n"
            + "} \n"
            + ".create-form input { \n"
            + "  border: 2px solid #294563; \n"
            + "  background-color: #f0f0ff; \n"
            + "  width: 300px; \n"
            + "} \n"
            + "select { \n"
            + "  font-size: 10px; \n"
            + "  background-color: #f0f0ff; \n"
            + "} \n"
            + ".upload-form input { \n"
            + "  border: 2px solid #294563; \n"
            + "  background-color: #f0f0ff; \n"
            + "} \n"
            + "--></style> \n"
            + "<title>";

    protected static final String END_HTML_HEADER_SECTION =
            "</title> \n" + "<body> \n\n";

    protected static final String LOGO_SECTION =
            "<table width=\"100%\"><tr> \n"
            + "<td width=\"100%\">&nbsp;</td> \n"
            + "<td align=\"right\" width=\"1px\"> \n"
            + "<div class=\"logo\"> xindice <div class=\"smallLogo\">(zeen-de-chay)</div> </div> \n"
            + "</td></tr></table> \n";

    protected static final String START_TABS_SECTION =
            "<table class=\"tabs\" cellspacing=\"0\" cellpadding=\"0\"> \n"
            + "  <tr> \n";

    protected static final String END_TABS_SECTION =
            "    <td width=\"1\" bgcolor=\"#808080\"><img width=\"1\" height=\"\" alt=\"\"></td> \n"
            + "  </tr>  \n"
            + "</table> \n";

    protected static final String START_TAB =
            "    <td class=\"unselected\">&nbsp;";

    protected static final String START_TAB_SELECTED =
            "    <td class=\"selected\">&nbsp;";

    protected static final String TAB_SPACE =
            "    <td width=\"1\" bgcolor=\"#808080\"><img width=\"1\" height=\"\" alt=\"\"></td> \n"
            + "    <td width=\"1\" bgcolor=\"#ffffff\"><img width=\"1\" height=\"1\" alt=\"\"></td> \n";

    protected static final String TAB_SPACE_SELECTED =
            "    <td width=\"1\" bgcolor=\"#808080\"><img width=\"1\" height=\"\" alt=\"\"></td> \n";

    protected static final String END_TAB = "&nbsp;</td> \n";

    protected static final String LEFT_PATH_SECTION =
            "<table border=\"0\" cellspacing=\"0\" cellpadding=\"3\" width=\"100%\"> \n"
            + "  <tr> \n"
            + "    <td class=\"misc\">&nbsp;";

    protected static final String RIGHT_PATH_SECTION =
            "</td> \n" + "    <td class=\"misc-right\">";

    protected static final String END_PATH_SECTION =
            "</td> \n" + "  </tr>  \n" + "</table> \n" + "<br /> \n";

    protected static final String START_TABLE =
            "<table cellspacing=\"0\" cellpadding=\"3\" class=\"contents\"> \n";

    protected static final String END_TABLE = "</table> \n";

    protected static final String START_TABLE_ROW = "<tr> \n";

    protected static final String END_TABLE_ROW = "</tr> \n";

    protected static final String START_TABLE_BOX = "<td> \n";

    protected static final String END_TABLE_BOX = "</td> \n";

    protected static final String START_TABLE_BOX_TITLE =
            "<td class=\"title\"> \n";

    protected static final String FOOTER =
            "\n<br /> \n"
            + "<table border=\"0\" cellspacing=\"0\" cellpadding=\"3\" width=\"100%\"> \n"
            + "  <tr> \n"
            + "    <td class=\"misc\">&nbsp; "
            + Xindice.Title
            + " "
            + Xindice.Version
            + "</td> \n"
            + "    <td class=\"misc-right\">&nbsp;</td> \n"
            + "  </tr> \n"
            + "</table>  \n";

    protected static final String HTML_TAIL_SECTION = "</body> \n" + "</html>";

    /* search form */

    protected static final String START_SEARCH_FORM =
            "<form action=\"\" method=\"get\"> \n"
            + "<table class=\"search-form\"> \n";

    protected static final String SEARCH_FROM_START_HIDDEN_VIEWER =
            "<input name=\"viewer\" value=\"";

    protected static final String SEARCH_FROM_END_HIDDEN_VIEWER =
            "\" type=\"hidden\" /> \n";

    protected static final String SEARCH_FROM_START_QUERY =
            "<tr><td colspan=\"3\"> \n"
            + "      query:  \n"
            + "      <input name=\"query\" type=\"text\" value=\"";

    protected static final String SEARCH_FROM_END_QUERY =
            "\" /><br /><div style=\"text-align:right;\">"
            + "<input type=\"submit\" name=\"submit\" value=\"search\" "
            + " style=\"margin-top:10px; width:auto;\"/> \n"
            + "</div><br /> \n"
            + "</td></tr> \n";

    protected static final String SEARCH_FROM_START_NS_HEADER =
            "<tr><td colspan=\"2\">optional namespaces:</td> \n"
            + "<td>number of ns: <select name=\"nssize\" size=\"1\"> \n";

    protected static final String SEARCH_FROM_START_NS_HEADER_OPT = "<option>";

    protected static final String SEARCH_FROM_START_NS_HEADER_OPT_SEL =
            "<option selected=\"selected\">";

    protected static final String SEARCH_FROM_END_NS_HEADER_OPT = "</option>";

    protected static final String SEARCH_FROM_END_NS_HEADER =
            "</select></td></tr>"
            + "<tr><td>&nbsp;</td><td>prefix</td><td>uri</td></tr> \n"
            + "<tr> \n";

    protected static final String SEARCH_FROM_START_NS = "<td>ns";

    protected static final String SEARCH_FROM_START_NS_PREFIX =
            ":</td> \n" + "<td><input name=\"prefix";

    protected static final String SEARCH_FROM_END_NS_PREFIX =
            "\" type=\"text\" value=\"";

    protected static final String SEARCH_FROM_START_NS_URI =
            "\"/></td> \n" + "<td><input name=\"uri";

    protected static final String SEARCH_FROM_MIDDLE_NS_URI =
            "\" type=\"text\" value=\"";

    protected static final String SEARCH_FROM_END_NS_URI =
            "\" /></td> \n" + "</tr> \n";

    protected static final String END_SEARCH_FORM =
            "</table> \n" + "</form> \n";

    protected static final String START_CONFIG_FORM =
            "<form action=\"?viewer=config\" method=\"post\"><textarea " +
            "name=\"config\" style=\"width:100%;\" rows=\"8\">";

    protected static final String END_CONFIG_FORM =
            "</textarea><br /><input type=\"submit\" name=\"submit\" value=\"update\"></form>";

    protected static final String CREATE_FORM_START =
            "<form action=\"\" method=\"get\"> \n"
            + "<input name=\"viewer\" value=\"create\" type=\"hidden\" /> \n"
            + "<table class=\"create-form\"> \n"
            + "<tr><td colspan=\"2\"> \n"
            + "	new collection name:  \n"
            + "	<input name=\"name\" type=\"text\" /><br /> \n"
            + "   <div style=\"text-align:right;\">"
            + "   <input type=\"submit\" name=\"submit\" value=\"create\" "
            + "   style=\"margin-top:10px; width:auto;\"/> \n"
            + "   </div><br />"
            + " </td></tr> \n"
            + " <tr><td colspan=\"2\">additional parameters:</td></tr> \n"
            + " <tr> \n"
            + "<td>enable Inline Metadata: </td> \n"
            + "<td> \n"
            + "  <select name=\"meta\" size=\"1\"> \n"
            + "	<option selected=\"selected\">";

    protected static final String CREATE_FORM_META_DEFAULT =
            "</option> \n" + "	<option>";

    protected static final String CREATE_FORM_META_END =
            "</option>       \n"
            + "  </select> \n"
            + "</td> \n"
            + " </tr> \n"
            + " <tr> \n"
            + "<td>compressed: </td> \n"
            + "<td> \n"
            + "  <select name=\"compressed\" size=\"1\"> \n"
            + "	<option selected=\"selected\">";

    protected static final String CREATE_FORM_COMPRESSED_DEFAULT =
            "</option> \n" + "	<option>";

    protected static final String CREATE_FORM_COMPRESSED_END =
            "</option>       \n"
            + "  </select> \n"
            + "</td> \n"
            + " </tr> \n"
            + " <tr> \n"
            + "<td>filer:</td> \n"
            + "<td><input name=\"filer\" type=\"text\" value=\"";

    protected static final String CREATE_FORM_FILER =
            "\"/></td> \n" + " </tr> \n" + " </table> \n" + " </form> \n";

    protected static final String UPLOAD_FORM =
            "<form action=\"\" method=\"post\" enctype=\"multipart/form-data\"> \n"
            + "<table class=\"upload-form\"> \n"
            + "<tr> \n"
            + "<td>upload file: </td> \n"
            + "<td><input name=\"file\" type=\"file\" /></td> \n"
            + "</td></tr> \n"
            + "<tr> \n"
            + "<td>name: </td> \n"
            + "<td><input name=\"name\" type=\"text\" /></td> \n"
            + "</td></tr> \n"
            + "<tr> \n"
            + "<td>type: </td>\n"
            + "<td><select name=\"type\" size=\"1\"> \n"
            + " <option>xml</option> \n"
            + " <option selected=\"selected\">detect</option> \n"
            + " <option>binary</option> \n"
            + "</select> \n"
            + "</td></tr> \n"
            + "<tr> \n"
            + "<td>overwrite: </td>\n"
            + "<td>no<input type=\"radio\" name=\"overwrite\" value=\"false\" checked=\"checked\" /> \n"
            + "yes<input type=\"radio\" name=\"overwrite\" value=\"true\" />       \n"
            + "</td></tr> \n"
            + "<tr><td> \n"
            + "<input type=\"submit\" name=\"submit\" value=\"upload\" /> \n"
            + "</td><td> "
            + "&nbsp; "
            + "</td></tr> \n"
            + "<tr><td colspan=\"2\"> \n"
            + "</td></tr> \n"
            + "</table> \n"
            + "</form> \n";
}
