/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.xindice.webadmin.webdav;

import org.apache.xindice.webadmin.PartialResponse;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletOutputStream;
import java.io.IOException;

/**
 * This is a wrapper class that provides additional DAV-specific functionality
 *
 * @version $Revision: 541515 $, $Date: 2007-05-24 22:45:06 -0400 (Thu, 24 May 2007) $
 */
public class DAVResponse {
    public static final String DAV_NS = "DAV:";

    protected final ServletOutputStream out;

    private boolean isMultistatus;
    private final HttpServletResponse response;
    private String protocol;

    private static final Log log = LogFactory.getLog(DAVResponse.class);

    /**
     * Creates new instance of DAVRequest.
     *
     * @param response HttpServletResponse
     */
    public DAVResponse(HttpServletResponse response) throws IOException {
        this.response = response;
        this.out = response.getOutputStream();
    }

    /**
     * For multi-status responses, send a partial response. This method resets
     * HTTP response status to 207 (Multi-Status).
     *
     * @param res partial response information
     * @throws IOException Servlet IOException
     */
    public void sendMultiStatusResponse(PartialResponse res) throws IOException {
        if (!isMultistatus) {
            if (log.isDebugEnabled()) {
                log.debug("Sending response for HREF " + res.getHref());
            }
            response.setStatus(WebdavStatus.SC_MULTI_STATUS);
            out.println("<?xml version='1.0' encoding='utf-8' ?>");
            out.println("<multistatus xmlns='DAV:'>");
            isMultistatus = true;
        }

        out.println(res.toString());
    }

    /**
     * For multi-status responses, close the response document. Method does not
     * affect not multi-status responses.
     * HTTP response status to 207 (Multi-Status).
     *
     * @throws IOException Servlet IOException
     */
    public void closeMultistatusResponse() throws IOException {
        if (isMultistatus) {
            out.println("</multistatus>");
        }
    }

    /**
     * Wrapper method, sets HTTP response status.
     * @param status
     */
    public void setStatus(int status) {
        response.setStatus(status);
    }

    /**
     * Wrapper method, adds header to HTTP response.
     * @param header Header name
     * @param value Header value
     */
    public void addHeader(String header, String value) {
        response.addHeader(header, value);
    }

    /**
     * Wrapper method, adds date header to HTTP response.
     * @param header Header name
     * @param date Header value
     */
    public void addDateHeader(String header, long date) {
        response.addDateHeader(header, date);
    }

    /**
     * Wrapper method, sets HTTP response content type
     * @param mimeType
     */
    public void setContentType(String mimeType) {
        response.setContentType(mimeType);
    }

    /**
     * Wrapper method, sets HTTP response content length
     * @param contentLength
     */
    public void setContentLength(int contentLength) {
        response.setContentLength(contentLength);
    }

    /**
     * Wrapper method, returns HTTP response output stream
     * @return Servlet output stream
     */
    public ServletOutputStream getOutputStream() throws IOException {
        return response.getOutputStream();
    }

    /**
     * Wrapper method, sends redirect
     * @param redirect
     */
    public void sendRedirect(String redirect) throws IOException {
        response.sendRedirect(redirect);
    }

    /**
     * Sets protocol/version string that is used for multi status responses.
     *
     * @param protocol Protocol name and version (e.g. HTTP/1.1)
     */
    public void setProtocol(String protocol) {
        this.protocol = protocol;
    }

    /**
     * Returns protocol/version.
     * @return Protocol name and version (e.g. HTTP/1.1)
     */
    public String getProtocol() {
        return protocol;
    }
}
