/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Id: TextWriter.java 541508 2007-05-25 01:54:12Z vgritsenko $
 */

package org.apache.xindice.xml;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xindice.util.StringUtilities;

import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.ProcessingInstruction;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.StringWriter;
import java.io.Writer;

/**
 * TextWriter takes a Document, DocumentFragment, or Element and
 * streams it as text into an {@link OutputStream}, {@link Writer},
 * or a {@link String}.
 *
 * UTF-8 output encoding is assumed.
 *
 * @version $Revision: 541508 $, $Date: 2007-05-24 21:54:12 -0400 (Thu, 24 May 2007) $
 */
public final class TextWriter {

    private static final Log log = LogFactory.getLog(TextWriter.class);

    private Node node;


    public TextWriter(Node node) {
        this.node = node;
    }

    /**
     * write writes the node to the writer as text.
     *
     * @param writer The Writer to write to
     */
    public void write(Writer writer) throws IOException {
        write(node, writer);
    }

    /**
     * write writes the node to the OutputStream as text.
     *
     * @param output The OutputStream to write to
     */
    public void write(OutputStream output) throws IOException {
        write(node, output);
    }

    /**
     * toString returns the node as a String.
     *
     * @return The String value
     */
    public String toString() {
        return toString(node);
    }

    private static void writeNode(Writer writer, Node node) throws IOException {
        short type = node.getNodeType();
        switch (type) {

            case Node.DOCUMENT_NODE:
                writer.write("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
                writeChildren(writer, node);
                break;

            case Node.DOCUMENT_FRAGMENT_NODE:
                writeChildren(writer, node);
                break;

            case Node.DOCUMENT_TYPE_NODE:
                // TODO Implement
                if (log.isErrorEnabled()) {
                    log.error("can't serialize doctype yet");
                }
                /*
                DocumentType d = (DocumentType)node;
                writer.write("<!DOCTYPE ");
                writer.write(node.getOwnerDocument().getDocumentElement().getNodeName());
                writer.write('>');
                */
                break;

            case Node.ELEMENT_NODE:
                Element e = (Element) node;
                String n = e.getTagName();

                writer.write('<');
                writer.write(n);

                NamedNodeMap a = e.getAttributes();
                int size = a.getLength();
                for (int i = 0; i < size; i++) {
                    Attr att = (Attr) a.item(i);
                    writer.write(' ');
                    writeNode(writer, att);
                }

                if (e.hasChildNodes()) {
                    writer.write('>');
                    writeChildren(writer, node);
                    writer.write("</");
                    writer.write(n);
                    writer.write('>');
                } else {
                    writer.write(" />");
                }
                break;

            case Node.ATTRIBUTE_NODE:
                Attr att = (Attr) node;
                writer.write(att.getName());
                writer.write("=\"");
                writer.write(StringUtilities.escapeXml(att.getValue()));
                writer.write("\"");
                break;

            case Node.ENTITY_REFERENCE_NODE:
                // TODO Implement
                if (log.isErrorEnabled()) {
                    log.error("can't serialize reference ref yet");
                }
                break;

            case Node.ENTITY_NODE:
                // TODO Implement
                if (log.isErrorEnabled()) {
                    log.error("can't serialize entity yet");
                }
                break;

            case Node.NOTATION_NODE:
                // TODO Implement
                if (log.isErrorEnabled()) {
                    log.error("can't serialize notation yet");
                }
                break;

            case Node.PROCESSING_INSTRUCTION_NODE:
                ProcessingInstruction pi = (ProcessingInstruction) node;
                writer.write("<?");
                writer.write(pi.getTarget());
                writer.write(" ");
                writer.write(pi.getData());
                writer.write("?>\n");
                break;

            case Node.TEXT_NODE:
                writer.write(StringUtilities.escapeXml(node.getNodeValue()));
                break;

            case Node.CDATA_SECTION_NODE:
                writer.write("<![CDATA[");
                writer.write(node.getNodeValue());
                writer.write("]]>");
                break;

            case Node.COMMENT_NODE:
                writer.write("<!--");
                writer.write(node.getNodeValue());
                writer.write("-->");
                break;

            default:
                if (log.isWarnEnabled()) {
                    log.warn("invalid node type : " + node);
                }
        }
    }

    private static void writeChildren(Writer writer, Node node) throws IOException {
        NodeList l = node.getChildNodes();
        int size = l.getLength();
        for (int i = 0; i < size; i++) {
            writeNode(writer, l.item(i));
        }
    }

    /**
     * write writes the specified node to the writer as text.
     *
     * @param node The Node to write
     * @param writer The Writer to write to
     */
    public static void write(Node node, Writer writer) throws IOException {
        BufferedWriter buf = new BufferedWriter(writer, 4096);
        writeNode(buf, node);
        buf.flush();
    }

    /**
     * write writes the specified node to the OutputStream as text.
     *
     * @param node The Node to write
     * @param output The OutputStream to write to
     */
    public static void write(Node node, OutputStream output) throws IOException {
        OutputStreamWriter o = new OutputStreamWriter(output, "utf-8");
        write(node, o);
    }

    /**
     * toString returns the node as a String.
     *
     * @param node The Node to convert
     * @return The String value
     */
    public static String toString(Node node) {
        StringWriter sw = new StringWriter();
        try {
            write(node, sw);
        } catch (IOException e) {
            // StringWriter does not generate any IOExceptions; it can create only OutOfMemoryError
        }
        return sw.toString();
    }
}
