/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Id: XMLCompressedOutput.java 541508 2007-05-25 01:54:12Z vgritsenko $
 */

package org.apache.xindice.xml;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.OutputStream;

/**
 * XMLCompressedInput is an OutputStream extension that provides functions for
 * writing types to a Xindice Compressed XML Stream.
 *
 * @version $Revision: 541508 $, $Date: 2007-05-24 21:54:12 -0400 (Thu, 24 May 2007) $
 */
public class XMLCompressedOutput extends DataOutputStream {

    protected SymbolTable st;


    public XMLCompressedOutput(OutputStream os, SymbolTable st) {
        super(os);
        this.st = st;
    }

    /**
     * getSizeType returns the optimal size type for the specifed
     * length value.
     *
     * @see org.apache.xindice.xml.Signatures
     *
     * @param length The length to evaluate
     * @return The optimal size
     */
    protected final byte getSizeType(long length) {
        if (length > Short.MAX_VALUE) {
            return Signatures.SIZE_INT;
        } else if (length > Byte.MAX_VALUE) {
            return Signatures.SIZE_SHORT;
        } else if (length > 0) {
            return Signatures.SIZE_BYTE;
        } else {
            return Signatures.SIZE_NONE;
        }
    }

    /**
     * getSizeSize returns the size (in bytes) of the specified
     * size type.
     *
     * @see org.apache.xindice.xml.Signatures
     *
     * @param sizeType The size type
     * @return The size in bytes
     */
    protected final int getSizeSize(int sizeType) {
        switch (sizeType) {
            case Signatures.SIZE_INT:
                return 4;
            case Signatures.SIZE_SHORT:
                return 2;
            case Signatures.SIZE_BYTE:
                return 1;
            default:
                return 0;
        }
    }

    /**
     * Writes the size to the output stream. Size type is determined
     * based on the passed size value.
     *
     *
     * @see org.apache.xindice.xml.Signatures
     *
     * @param size The size
     * @return The size in bytes written
     * @throws IOException If the write failed
     */
    protected final int writeSize(int size) throws IOException {
        return writeSize(getSizeType(size), size);
    }

    /**
     * writeSize writes the size to the output stream based on the
     * specified sizeType.
     *
     * @see org.apache.xindice.xml.Signatures
     *
     * @param sizeType The size type
     * @param size The size
     * @return The size in bytes written
     * @throws IOException If the write failed
     */
    protected final int writeSize(int sizeType, int size) throws IOException {
        switch (sizeType) {

            case Signatures.SIZE_INT:
                writeInt(size);
                return 4;

            case Signatures.SIZE_SHORT:
                writeShort((short) size);
                return 2;

            case Signatures.SIZE_BYTE:
                writeByte((byte) size);
                return 1;

            default:
                return 0;
        }
    }
}
