/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Id: Xml2HtmlWriter.java 541508 2007-05-25 01:54:12Z vgritsenko $
 */

package org.apache.xindice.xml;


import org.w3c.dom.Node;
import org.w3c.dom.DOMException;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Attr;
import org.w3c.dom.ProcessingInstruction;
import org.w3c.dom.NodeList;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.logging.Log;
import org.apache.xindice.util.StringUtilities;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;

/**
 * Xml2HtmlWriter takes a Document, DocumentFragment, or Element and
 * streams it as html to an output source (or a String).
 * <p/>
 * a css stylesheet should define these classes:
 * xml-misc, xml-element-name, xml-attribute-name, xml-attribute-content,
 * xml-element-content, xml-comment, xml-processing, xml-cdata
 *
 * @author <a href="mailto:jmetzner@apache.org">Jan Metzner</a>
 * @version $Id: Xml2HtmlWriter.java 541508 2007-05-25 01:54:12Z vgritsenko $
 */
public class Xml2HtmlWriter {

    private static final String indent = "&nbsp;&nbsp;";
    private Node node = null;
    private static final Log log = LogFactory.getLog(Xml2HtmlWriter.class);

    public Xml2HtmlWriter(Node node) throws DOMException {
        this.node = node;
    }

    /**
     * write writes the node to the writer as text.
     *
     * @param writer The Writer to write to
     */
    public void write(Writer writer) throws IOException {
        write(node, writer);
    }

    /**
     * write writes the node to the OutputStream as text.
     *
     * @param output The OutputStream to write to
     */
    public void write(OutputStream output) throws IOException {
        write(node, output);
    }

    /**
     * toString returns the node as a String.
     *
     * @return The String value
     */
    public String toString() {
        return toString(node);
    }

    /**
     * writes the Node to the writer.
     *
     * @param writer        The Writer to write to.
     * @param node          The Node to write.
     * @param currentIndent the current indent.
     * @throws IOException while writing to the writer.
     */
    private static void writeNode(Writer writer, Node node, int currentIndent) throws IOException {
        short type = node.getNodeType();

        switch (type) {
            case Node.DOCUMENT_NODE: {
                writeIndent(writer, currentIndent);
                writer.write("<span class=\"xml-processing\">&lt;?xml ");
                writer.write(" version=\"1.0\"?&gt;</span><br />\n");
                currentIndent--;
                writeChildren(writer, node, currentIndent);
                break;
            }

            case Node.DOCUMENT_FRAGMENT_NODE: {
                writeChildren(writer, node, currentIndent);
                break;
            }

            case Node.DOCUMENT_TYPE_NODE: {
                break;
            }

            case Node.ELEMENT_NODE: {
                Element e = (Element) node;
                String n = e.getTagName();

                writeIndent(writer, currentIndent);
                writer.write("<span class=\"xml-misc\">&lt;</span>");
                writer.write("<span class=\"xml-element-name\">");
                writer.write(n);
                writer.write("</span>");

                NamedNodeMap a = e.getAttributes();
                int size = a.getLength();
                for (int i = 0; i < size; i++) {
                    Attr att = (Attr) a.item(i);
                    writer.write(' ');
                    writeNode(writer, att, currentIndent);
                }

                if (e.hasChildNodes()) {
                    writer.write("<span class=\"xml-misc\">&gt;</span><br />\n");
                    writeChildren(writer, node, currentIndent);
                    writeIndent(writer, currentIndent);
                    writer.write("<span class=\"xml-misc\">&lt;/</span>");
                    writer.write("<span class=\"xml-element-name\">");
                    writer.write(n);
                    writer.write("</span><span class=\"xml-misc\">&gt;</span><br />\n");
                } else {
                    writer.write("<span class=\"xml-misc\"> /&gt;</span><br />\n");
                }
                break;
            }

            case Node.ATTRIBUTE_NODE: {
                Attr a = (Attr) node;
                writer.write("<span class=\"xml-attribute-name\">");
                writer.write(a.getName());
                writer.write("</span><span class=\"xml-misc\">=\"</span>");
                writer.write("<span class=\"xml-attribute-content\">");
                writeEscapedText(writer, a.getValue(), currentIndent);
                writer.write("</span><span class=\"xml-misc\">\"</span>");
                break;
            }

            case Node.ENTITY_REFERENCE_NODE: {
                break;
            }

            case Node.ENTITY_NODE: {
                break;
            }

            case Node.NOTATION_NODE: {
                break;
            }

            case Node.PROCESSING_INSTRUCTION_NODE: {
                ProcessingInstruction pi = (ProcessingInstruction) node;
                writeIndent(writer, currentIndent);
                writer.write("<span class=\"xml-processing\">&lt;?");
                writer.write(pi.getTarget());
                writer.write(" ");
                writer.write(pi.getData());
                writer.write("?&gt;</span><br />\n");
                break;
            }

            case Node.TEXT_NODE: {
                if (!StringUtilities.isBlank(node.getNodeValue())) {
                    writeIndent(writer, currentIndent);
                    writer.write("<span class=\"xml-element-content\">");
                    writeEscapedText(writer, node.getNodeValue().trim(), currentIndent);
                    writer.write("</span>");
                }
                break;
            }

            case Node.CDATA_SECTION_NODE: {
                writeIndent(writer, currentIndent);
                writer.write("<span class=\"xml-cdata\">&lt;![CDATA[");
                writeEscapedText(writer, node.getNodeValue(), currentIndent);
                writer.write("]]&gt;</span><br />");
                break;
            }

            case Node.COMMENT_NODE: {
                writeIndent(writer, currentIndent);
                writer.write("<span class=\"xml-comment\">&lt;!--");
                writeEscapedText(writer, node.getNodeValue(), currentIndent);
                writer.write("--&gt;</span><br />");
                break;
            }
        }
    }

    /**
     * writes the children nodes for the given node.
     *
     * @param writer        The Writer to write to.
     * @param node          The Parent Node.
     * @param currentIndent The current Indent.
     * @throws IOException while writing to the writer.
     */
    private static void writeChildren(Writer writer, Node node, int currentIndent) throws IOException {
        NodeList l = node.getChildNodes();
        int size = l.getLength();
        currentIndent++;
        for (int i = 0; i < size; i++) {
            writeNode(writer, l.item(i), currentIndent);
        }
    }

    /**
     * writes the text as escaped html.
     *
     * @param writer        The Writer to write to.
     * @param text          The text to write.
     * @param currentIndent The current Indent.
     * @throws IOException
     */
    private static void writeEscapedText(Writer writer, String text, int currentIndent) throws IOException {
        char[] value = text.toCharArray();
        String outval = null;
        int start = 0;
        int len = 0;
        for (int i = 0; i < value.length; i++) {
            switch (value[i]) {
                case '&':
                    outval = "&amp;";
                    break;
                case '\'':
                    outval = "&apos;";
                    break;
                case '\"':
                    outval = "&quot;";
                    break;
                case '<':
                    outval = "&lt;";
                    break;
                case '>'  :
                    outval = "&gt;";
                    break;
                case '\n':
                    //writer.write("<br />");
                    //writeIndent(writer, currentIndent);
                    outval = "<br />";
                    for (int i1 = 0; i1 < currentIndent; i1++) {
                        outval += indent;
                    }
                    break;
                default:
                    len++;
                    break;
            }

            if (outval != null) {
                if (len > 0) {
                    writer.write(value, start, len);
                }
                writer.write(outval);
                start = i + 1;
                len = 0;
                outval = null;
            }
        }
        if (len > 0) {
            writer.write(value, start, len);
        }
    }

    /**
     * writes the indent to the writer.
     *
     * @param writer        The Writer to write to.
     * @param currentIndent The current Indent.
     * @throws IOException
     */
    private static void writeIndent(Writer writer, int currentIndent)
            throws IOException {
        for (int i = 0; i < currentIndent; i++) {
            writer.write(indent);
        }
    }

    /**
     * write writes the specified node to the writer as text.
     *
     * @param node   The Node to write.
     * @param writer The Writer to write to.
     */
    public static void write(Node node, Writer writer) throws IOException {
        BufferedWriter buf = new BufferedWriter(writer, 4096);
        writeNode(buf, node, 0);
        buf.flush();
    }

    /**
     * write writes the specified node to the OutputStream as text.
     *
     * @param node   The Node to write.
     * @param output The OutputStream to write to.
     */
    public static void write(Node node, OutputStream output) throws IOException {
        try {
            OutputStreamWriter o = new OutputStreamWriter(output, "utf-8");
            write(node, o);
        } catch (UnsupportedEncodingException e) {
            log.error(e); //won't happen
        }
    }

    /**
     * toString returns the node as a String.
     *
     * @param node The Node to convert
     * @return The String value
     */
    public static String toString(Node node) {
        StringWriter sw = new StringWriter();
        try {
            write(node, sw);
            return sw.toString();
        } catch (IOException e) {
            log.error(e);
            return null;
                }
        }

}
