/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Id: AttrImpl.java 571948 2007-09-02 10:51:37Z vgritsenko $
 */

package org.apache.xindice.xml.dom;

import org.w3c.dom.Attr;
import org.w3c.dom.DOMException;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.TypeInfo;
import org.w3c.dom.UserDataHandler;

/**
 * AttrImpl
 *
 * @version $Revision: 571948 $, $Date: 2007-09-02 06:51:37 -0400 (Sun, 02 Sep 2007) $
 */
public final class AttrImpl extends ContainerNodeImpl
                            implements Attr {

    private boolean specified = true;
    private short symbolID = -1;


    public AttrImpl() {
    }

    public AttrImpl(NodeImpl parent, byte[] data, int pos, int len) {
        super(parent, data, pos, len);
    }

    public AttrImpl(NodeImpl parent, String nodeName, String nsURI, short symbolID, String nodeValue) {
        super(parent, false);
        this.nodeName = nodeName;
        this.nsURI = nsURI;
        this.symbolID = symbolID;
        this.nodeValue = nodeValue;

        TextImpl text = new TextImpl(this, false);
        text.nodeValue = nodeValue;
        childNodes.add(text);
    }

    public AttrImpl(NodeImpl parent, boolean dirty) {
        super(parent, dirty);
    }

    public AttrImpl(NodeImpl parent, String nodeName) {
        super(parent, true);
        this.nodeName = nodeName;
    }

    public short getNodeType() {
        return Node.ATTRIBUTE_NODE;
    }

    /**
     * Returns the name of this attribute.
     */
    public String getName() {
        return getNodeName();
    }

    public void setSymbolID(short symbolID) {
        this.symbolID = symbolID;
    }

    public short getSymbolID() {
        return symbolID;
    }

    public void setNodeValue(String nodeValue) throws DOMException {
        checkLoaded();
        checkReadOnly();

        childNodes.clear();
        TextImpl text = new TextImpl(this, false);
        text.nodeValue = nodeValue;
        childNodes.add(text);
        setDirty();
    }

    public void setValue(String value) {
        setNodeValue(value);
    }

    protected boolean isNodeTypeValid(short type) {
        return type == Node.TEXT_NODE ||
               type == Node.ENTITY_REFERENCE_NODE;
    }

    /**
     * If this attribute was explicitly given a value in the original document,
     * this is <code>true</code>; otherwise, it is <code>false</code>. Note
     * that the implementation is in charge of this attribute, not the user. If
     * the user changes the value of the attribute (even if it ends up having
     * the same value as the default value) then the <code>specified</code>
     * flag is automatically flipped to <code>true</code>.  To re-specify the
     * attribute as the default value from the DTD, the user must delete the
     * attribute. The implementation will then make a new attribute available
     * with <code>specified</code> set to <code>false</code> and the default
     * value (if one exists).
     * <br>In summary: If the attribute has an assigned value in the document
     * then  <code>specified</code> is <code>true</code>, and the value is the
     * assigned value. If the attribute has no assigned value in the document
     * and has  a default value in the DTD, then <code>specified</code> is
     * <code>false</code>,  and the value is the default value in the DTD. If
     * the attribute has no assigned value in the document and has  a value of
     * #IMPLIED in the DTD, then the  attribute does not appear  in the
     * structure model of the document.
     */
    public boolean getSpecified() {
        checkLoaded();
        return specified;
    }

    public void setSpecified(boolean specified) {
        checkLoaded();
        this.specified = specified;
    }

    /**
     * On retrieval, the value of the attribute is returned as a string.
     * Character and general entity references are replaced with their values.
     * <br>
     * On setting, this creates a <code>Text</code> node with the unparsed
     * contents of the string.
     */
    public String getValue() {
        return getNodeValue();
    }

    public String getNodeValue() {
        StringBuffer sb = new StringBuffer();
        int size = childNodes.getLength();
        for (int i = 0; i < size; i++) {
            sb.append(childNodes.item(i).getNodeValue());
        }

        return sb.toString();
    }

    /**
     * @since DOM Level 2
     */
    public Element getOwnerElement() {
        if (parentNode.getNodeType() == Node.ELEMENT_NODE) {
            return (Element) parentNode;
        } else {
            return null;
        }
    }

    //
    // DOM Level 3 Implementation
    //

    /**
     * @since DOM Level 3
     */
    public TypeInfo getSchemaTypeInfo() {
        return null;
    }

    /**
     * @since DOM Level 3
     */
    public boolean isId() {
        return false;
    }

    Node renameNode(String namespaceURI, String qualifiedName, String prefix) {
        checkLoaded();
        checkReadOnly();

        if ((XMLNS_PREFIX.equals(prefix) || XMLNS_PREFIX.equals(qualifiedName)) &&
                !XMLNS_URI.equals(namespaceURI)) {
            throw new DOMException(DOMException.NAMESPACE_ERR, XMLNS_PREFIX + " prefix is reserved.");
        }

        nodeName = qualifiedName;
        if (prefix != null) {
            getOwnerElement().setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:" + prefix, namespaceURI);
        }

        invokeHandlers(UserDataHandler.NODE_RENAMED, this, null);
        return this;
    }
}
