/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Id: TreeWalkerImpl.java 541508 2007-05-25 01:54:12Z vgritsenko $
 */

package org.apache.xindice.xml.dom.traversal;

import org.apache.xindice.xml.dom.NodeImpl;

import org.w3c.dom.DOMException;
import org.w3c.dom.Node;
import org.w3c.dom.traversal.NodeFilter;
import org.w3c.dom.traversal.NodeIterator;
import org.w3c.dom.traversal.TreeWalker;

/**
 * TreeWalkerImpl
 *
 * @version $Revision: 541508 $, $Date: 2007-05-24 21:54:12 -0400 (Thu, 24 May 2007) $
 */
public final class TreeWalkerImpl implements TreeWalker, NodeIterator {
    protected boolean valid;
    protected Node root;
    protected Node current;
    protected int whatToShow;
    protected NodeFilter filter;
    protected boolean expand;


    public TreeWalkerImpl(Node root, int whatToShow, NodeFilter filter, boolean entityReferenceExpansion) throws DOMException {
        this.valid = true;
        this.root = root;
        this.current = root;
        this.whatToShow = whatToShow;
        this.filter = filter;
        this.expand = entityReferenceExpansion;
    }

    private int getShowType(Node node) {
        switch (node.getNodeType()) {
            case Node.ATTRIBUTE_NODE:
                return NodeFilter.SHOW_ATTRIBUTE;
            case Node.CDATA_SECTION_NODE:
                return NodeFilter.SHOW_CDATA_SECTION;
            case Node.COMMENT_NODE:
                return NodeFilter.SHOW_COMMENT;
            case Node.DOCUMENT_FRAGMENT_NODE:
                return NodeFilter.SHOW_DOCUMENT_FRAGMENT;
            case Node.DOCUMENT_NODE:
                return NodeFilter.SHOW_DOCUMENT;
            case Node.DOCUMENT_TYPE_NODE:
                return NodeFilter.SHOW_DOCUMENT_TYPE;
            case Node.ELEMENT_NODE:
                return NodeFilter.SHOW_ELEMENT;
            case Node.ENTITY_NODE:
                return NodeFilter.SHOW_ENTITY;
            case Node.ENTITY_REFERENCE_NODE:
                return NodeFilter.SHOW_ENTITY_REFERENCE;
            case Node.NOTATION_NODE:
                return NodeFilter.SHOW_NOTATION;
            case Node.PROCESSING_INSTRUCTION_NODE:
                return NodeFilter.SHOW_PROCESSING_INSTRUCTION;
            case Node.TEXT_NODE:
                return NodeFilter.SHOW_TEXT;
            default:
                return 0;
        }
    }

    private boolean acceptNode(Node node) {
        return ((getShowType(node) & whatToShow) > 0)
                && (filter == null || filter.acceptNode(node) == NodeFilter.FILTER_ACCEPT);
    }

    /**
     * The root node of the Iterator, as specified when it was created.
     */
    public Node getRoot() {
        return root;
    }

    /**
     * This attribute determines which node types are presented via the
     * iterator. The available set of constants is defined in the
     * <code>NodeFilter</code> interface.
     */
    public int getWhatToShow() {
        return whatToShow;
    }

    /**
     * The filter used to screen nodes.
     */
    public NodeFilter getFilter() {
        return filter;
    }

    /**
     * The value of this flag determines whether the children of entity
     * reference nodes are visible to the iterator. If false, they will be
     * skipped over.
     *
     * <br>To produce a view of the document that has entity references
     * expanded and does not expose the entity reference node itself, use the
     * whatToShow flags to hide the entity reference node and set
     * expandEntityReferences to true when creating the iterator. To produce
     * a view of the document that has entity reference nodes but no entity
     * expansion, use the whatToShow flags to show the entity reference node
     * and set expandEntityReferences to false.
     */
    public boolean getExpandEntityReferences() {
        return expand;
    }

    /**
     * Detaches the iterator from the set which it iterated over, releasing
     * any computational resources and placing the iterator in the INVALID
     * state. After<code>detach</code> has been invoked, calls to
     * <code>nextNode</code> or<code>previousNode</code> will raise the
     * exception INVALID_STATE_ERR.
     */
    public void detach() {
        valid = false;
    }

    /**
     * The node at which the TreeWalker is currently positioned.
     *
     * <br>The value must not be null. Alterations to the DOM tree may cause
     * the current node to no longer be accepted by the TreeWalker's
     * associated filter. currentNode may also be explicitly set to any node,
     * whether or not it is within the subtree specified by the root node or
     * would be accepted by the filter and whatToShow flags. Further
     * traversal occurs relative to currentNode even if it is not part of the
     * current view by applying the filters in the requested direction (not
     * changing currentNode where no traversal is possible).
     *
     * @exception DOMException
     *    NOT_SUPPORTED_ERR: Raised if the specified <code>currentNode</code>
     *   is<code>null</code> .
     */
    public Node getCurrentNode() {
        return current;
    }

    public void setCurrentNode(Node currentNode) throws DOMException {
        if (!valid) {
            throw NodeImpl.EX_INVALID_STATE;
        }
        if (currentNode == null) {
            throw new DOMException(DOMException.NOT_SUPPORTED_ERR,
                                   "Current node can not be set to null.");
        }
        this.current = currentNode;
    }

    /**
     * Moves to and returns the closest visible ancestor node of the current
     * node. If the search for parentNode attempts to step upward from the
     * TreeWalker's root node, or if it fails to find a visible ancestor
     * node, this method retains the current position and returns null.
     *
     * @return  The new parent node, or null if the current node has no parent
     *          in the TreeWalker's logical view.
     */
    public Node parentNode() {
        Node node = this.current;
        while (node != null) {
            if (node == root) {
                return null;
            }
            node = node.getParentNode();
            if (node == null) {
                return null;
            } else if (acceptNode(node)) {
                this.current = node;
                return this.current;
            }
        }
        return null;
    }

    /**
     * Moves the <code>TreeWalker</code> to the first visible child of the
     * current node, and returns the new node. If the current node has no
     * visible children, returns <code>null</code> , and retains the current
     * node.
     *
     * @return  The new node, or <code>null</code> if the current node has no
     *          visible children in the TreeWalker's logical view.
     */
    public Node firstChild() {
        Node node = this.current.getFirstChild();
        while (node != null) {
            if (acceptNode(node)) {
                this.current = node;
                return this.current;
            }

            node = node.getNextSibling();
        }
        return null;
    }

    /**
     * Moves the <code>TreeWalker</code> to the last visible child of the
     * current node, and returns the new node. If the current node has no
     * visible children, returns <code>null</code> , and retains the current
     * node.
     *
     * @return  The new node, or <code>null</code> if the current node has no
     *          children  in the TreeWalker's logical view.
     */
    public Node lastChild() {
        Node node = this.current.getLastChild();
        while (node != null) {
            if (acceptNode(node)) {
                this.current = node;
                return this.current;
            }

            node = node.getPreviousSibling();
        }
        return null;
    }

    /**
     * Moves the <code>TreeWalker</code> to the previous sibling of the
     * current node, and returns the new node. If the current node has no
     * visible previous sibling, returns <code>null</code> , and retains the
     * current node.
     *
     * @return  The new node, or <code>null</code> if the current node has no
     *          previous sibling in the TreeWalker's logical view.
     */
    public Node previousSibling() {
        Node node = this.current.getPreviousSibling();
        while (node != null) {
            if (acceptNode(node)) {
                this.current = node;
                return this.current;
            }

            node = node.getPreviousSibling();
        }
        return null;
    }

    /**
     * Moves the <code>TreeWalker</code> to the next sibling of the current
     * node, and returns the new node. If the current node has no visible
     * next sibling, returns <code>null</code> , and retains the current node.
     *
     * @return  The new node, or <code>null</code> if the current node has no
     *          next sibling in the TreeWalker's logical view.
     */
    public Node nextSibling() {
        Node node = this.current.getNextSibling();
        while (node != null) {
            if (acceptNode(node)) {
                this.current = node;
                return this.current;
            }

            node = node.getNextSibling();
        }
        return null;
    }

    /**
     * Moves the <code>TreeWalker</code> to the previous visible node in
     * document order relative to the current node, and returns the new node.
     * If the current node has no previous node,  or if the search for
     * previousNode attempts to step upward from the TreeWalker's root node,
     * returns <code>null</code> , and retains the current node.
     *
     * @return  The new node, or <code>null</code> if the current node has no
     *          previous node in the TreeWalker's logical view.
     */
    public Node previousNode() {
        Node node = this.current;
        while (true) {
            Node tmp = node.getPreviousSibling();
            if (tmp == null) {
                if (node == root) {
                    // Do not go out of the root
                    return null;
                }

                tmp = node.getParentNode();
            } else {
                while (tmp.hasChildNodes()) {
                    tmp = tmp.getLastChild();
                }
            }
            node = tmp;

            if (node == null) {
                return null;
            } else if (acceptNode(node)) {
                this.current = node;
                return this.current;
            }
        }
    }

    /**
     * Moves the <code>TreeWalker</code> to the next visible node in document
     * order relative to the current node, and returns the new node. If the
     * current node has no next node,  or if the search for nextNode attempts
     * to step upward from the TreeWalker's root node, returns
     * <code>null</code> , and retains the current node.
     *
     * @return  The new node, or <code>null</code> if the current node has no
     *          next node  in the TreeWalker's logical view.
     */
    public Node nextNode() {
        Node node = this.current;
        while (true) {
            if (node.hasChildNodes()) {
                node = node.getFirstChild();
            } else {
                while (true) {
                    Node old = node;
                    node = node.getNextSibling();
                    if (node == null) {
                        node = old;
                        node = node.getParentNode();
                        if (node == null || node == root) {
                            // this.next stays the same
                            return null;
                        }
                    } else {
                        break;
                    }
                }
            }
            if (node != null && acceptNode(node)) {
                this.current = node;
                return node;
            }
        }
    }
}
