/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Id: LuceneIndexerTest.java 568372 2007-08-22 01:58:31Z natalia $
 */

package org.apache.xindice.core.indexer;

import junit.framework.TestCase;
import org.apache.xindice.core.Database;
import org.apache.xindice.core.Collection;
import org.apache.xindice.core.DatabaseTest;
import org.apache.xindice.util.Configuration;
import org.apache.xindice.xml.dom.DOMParser;
import org.w3c.dom.Document;

import java.util.HashMap;
import java.util.Iterator;

/**
 * Tests indexed queries
 *
 * @version $Revision: 568372 $, $Date: 2007-08-21 21:58:31 -0400 (Tue, 21 Aug 2007) $
 */
public class LuceneIndexerTest extends TestCase {

    private Database db;
    private Collection collection;
    protected String indexClass;


    public LuceneIndexerTest(String name) {
        super(name);
        indexClass = "org.apache.xindice.core.indexer.LuceneIndexer";
    }

    public void setUp() throws Exception {
        String name = getClass().getName();
        db = new Database();
        db.setConfig(new Configuration(DOMParser.toDocument(DatabaseTest.DATABASE)));
        collection = db.createCollection(name, new Configuration(
                DOMParser.toDocument(
                        "<collection compressed='true' name='" + name + "' inline-metadata='true'>" +
                        "<filer class='org.apache.xindice.core.filer.BTreeFiler'/>" +
                        "</collection>"), false
        ));

        Document document = DOMParser.toDocument("<doc><a>test</a> <b>foo</b></doc>");
        collection.insertDocument("key1", document);

        document = DOMParser.toDocument("<doc><a>test</a> <b>bar</b></doc>");
        collection.insertDocument("key2", document);

        document = DOMParser.toDocument("<doc><a>test</a> <b>test</b></doc>");
        collection.insertDocument("key3", document);

        document = DOMParser.toDocument("<doc><a>text</a> <b>test</b></doc>");
        collection.insertDocument("key4", document);

        document = DOMParser.toDocument("<doc><a>foo</a> <b>test</b></doc>");
        collection.insertDocument("key5", document);
    }

    public void tearDown() throws Exception {
        db.dropCollection(collection);
        db.close();
    }

    private LuceneIndexer createIndex(String name, HashMap patterns, String defaultField) throws Exception {
        String config = "<index name='" + name + "' " +
                        "class='" + indexClass + "' " +
                        "analyzer='org.apache.lucene.analysis.SimpleAnalyzer'>";
        for (Iterator i = patterns.keySet().iterator(); i.hasNext(); ) {
            String alias = (String) i.next();
            config += "<pattern pattern='" + patterns.get(alias) + "' alias='" + alias + "' />";
        }
        if (defaultField != null) {
            config += "<default alias='" + defaultField + "' />";
        }
        config += "</index>";

        LuceneIndexer ind = (LuceneIndexer) collection.createIndexer(new Configuration(DOMParser.toDocument(config)));
        Thread.sleep(1000);
        return ind;
    }

    private IndexMatch[] query(Indexer idx, String query) throws Exception {
        return idx.queryMatches(new IndexQuery(null, IndexQuery.TQ, query));
    }

    public void testSingleIndex() throws Exception {
        HashMap patterns = new HashMap();
        patterns.put("test", "a");
        LuceneIndexer idx = createIndex("test", patterns, null);

        String query = "test:test";
        IndexMatch[] match = query(idx, query);

        assertEquals(3, match.length);
    }

    public void testDuplicate() throws Exception {
        try {
            HashMap patterns = new HashMap();
            patterns.put("test1", "a@*");
            patterns.put("test2", "b@*");
            createIndex("test1", patterns, null);
            createIndex("test2", patterns, null);
            fail("There can be only one full text index per collection");
        } catch (DuplicateIndexException e) {
            // correct behavior
        }
    }

    public void testMultipleIndexes() throws Exception {
        HashMap patterns = new HashMap();
        patterns.put("test1", "a@*");
        patterns.put("test2", "b@*");
        LuceneIndexer idx = createIndex("test", patterns, "test1");

        IndexMatch[] match = query(idx, "test");
        assertEquals(3, match.length);

        match = query(idx, "test2:test");
        assertEquals(3, match.length);

        match = query(idx, "test AND test2:test");
        assertEquals(1, match.length);

        match = query(idx, "test OR test2:test");
        assertEquals(5, match.length);
    }

    public void testNestedContent() throws Exception {
        HashMap patterns = new HashMap();
        patterns.put("doc", "doc");
        LuceneIndexer idx = createIndex("test", patterns, "doc");

        String query = "foo";
        IndexMatch[] match = query(idx, query);

        assertEquals(2, match.length);
    }
}
