/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Id: IndexedQueryTest.java 585301 2007-10-17 00:37:44Z natalia $
 */
package org.apache.xindice.core.query;

import org.apache.xindice.core.Collection;
import org.apache.xindice.core.Database;
import org.apache.xindice.core.DatabaseTest;
import org.apache.xindice.core.data.NodeSet;
import org.apache.xindice.util.Configuration;
import org.apache.xindice.xml.dom.DOMParser;

import junit.framework.TestCase;
import org.w3c.dom.Document;

/**
 * Tests indexed queries
 *
 * @version $Revision: 585301 $, $Date: 2007-10-16 20:37:44 -0400 (Tue, 16 Oct 2007) $
 */
public class IndexedQueryTest extends TestCase {

    private Database db;
    private Collection collection;


    public IndexedQueryTest(String name) {
        super(name);
    }

    public void setUp() throws Exception {
        String name = getClass().getName();
        db = new Database();
        db.setConfig(new Configuration(DOMParser.toDocument(DatabaseTest.DATABASE)));
        collection = db.createCollection(name, new Configuration(
                DOMParser.toDocument(
                        "<collection compressed='true' name='" + name + "' inline-metadata='true'>" +
                        "  <filer class='org.apache.xindice.core.filer.BTreeFiler'/>" +
                        "</collection>"), false
        ));

        String config = "<index name='Test' " +
                        "       class='org.apache.xindice.core.indexer.ValueIndexer' " +
                        "       pattern='test1'/>";
        collection.createIndexer(new Configuration(DOMParser.toDocument(config)));
    }

    public void tearDown() throws Exception {
        db.dropCollection(collection);
        db.close();
    }

    public void testStartsWithAndLTGTQuery() throws Exception {
        int iterations = 10;
        for (int i = 0; i < iterations; i++) {
            Document document = DOMParser.toDocument("<test string='true'><test1>" + i +
                                                     "</test1>This is a test document</test>");
            collection.insertDocument("key" + i, document);
        }

        assertTrue(collection.getFiler().getRecordCount() == iterations);

        XPathQueryResolver service = new XPathQueryResolver();
        NodeSet result = service.query(collection, "//test1[starts-with(., '8')]", null, null);
        int count = 0;
        while (result.hasMoreNodes()) {
            result.getNextNode();
            count++;
        }

        assertEquals(1, count);

        result = service.query(collection, "//test1[. < '8' or . > '8']", null, null);
        count = 0;
        while (result.hasMoreNodes()) {
            result.getNextNode();
            count++;
        }

        assertEquals(9, count);
    }

    /**
     * Tests specific case with more complex location path to check if
     * XPathQueryResolver picks the correct index
     * @throws Exception
     */
    public void testStartsWithQuery() throws Exception {
        String config = "<index name='SpecialTest' " +
                        "       class='org.apache.xindice.core.indexer.ValueIndexer' " +
                        "       pattern='test1@num'/>";
        collection.createIndexer(new Configuration(DOMParser.toDocument(config)));

        int iterations = 10;
        for (int i = 0; i < iterations; i++) {
            Document document = DOMParser.toDocument("<test num='" + (i + 1) + "'>" +
                                                     "<test1 num='" + i + "'/>" +
                                                     "This is a test document</test>");
            collection.insertDocument("key" + i, document);
        }

        assertTrue(collection.getFiler().getRecordCount() == iterations);

        XPathQueryResolver service = new XPathQueryResolver();
        NodeSet result = service.query(collection, "//test1[starts-with(../@num, '8')]", null, null);
        int count = 0;
        while (result.hasMoreNodes()) {
            result.getNextNode();
            count++;
        }

        assertEquals(1, count);
    }

    public void testValueIndexer_StartsWithNestedContent() throws Exception {
        collection.insertDocument("key1", DOMParser.toDocument("<test1>This <i>is</i> a test</test1>"));
        collection.insertDocument("key2", DOMParser.toDocument("<test2>This <i>is</i> a test</test2>"));
        assertEquals(2, collection.getFiler().getRecordCount());

        XPathQueryResolver service = new XPathQueryResolver();
        NodeSet result = service.query(collection, "//test2[starts-with(., 'This is')]", null, null);
        int count = 0;
        while (result.hasMoreNodes()) {
            result.getNextNode();
            count++;
        }
        assertEquals(1, count);

        result = service.query(collection, "//test1[starts-with(., 'This is')]", null, null);
        count = 0;
        while (result.hasMoreNodes()) {
            result.getNextNode();
            count++;
        }
        assertEquals(1, count);

    }
}
