/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Id: SymbolTableTest.java 568786 2007-08-23 00:40:57Z vgritsenko $
 */

package org.apache.xindice.xml;

import org.apache.xindice.util.ReadOnlyException;
import org.apache.xindice.xml.dom.DOMParser;

import junit.framework.TestCase;

/**
 * @version $Revision: 568786 $, $Date: 2007-08-22 20:40:57 -0400 (Wed, 22 Aug 2007) $
 */
public class SymbolTableTest extends TestCase {

    private static final String SYMBOLS =
            "<symbols>" +
            "  <symbol id='1' name='simple'/>" +
            "  <symbol id='2' nsuri='http://www.w3.org/1999/xhtml' name='spaced'/>" +
            "  <symbol id='3' nsuri='http://www.w3.org/1999/xhtml' name='a:spaced'/>" +
            "  <symbol id='4' nsuri='http://www.w3.org/1999/xhtml' name='b:spaced'/>" +
            "</symbols>";

    private SymbolTable symbols;


    protected void setUp() throws Exception {
        symbols = new SymbolTable(DOMParser.toDocument(SYMBOLS).getDocumentElement());
    }


    public void testMissingSymbol() throws Exception {
        assertNull(symbols.getSymbolInfo((short) 123));

        assertEquals(-1, symbols.getSymbol("missing"));
        assertEquals(-1, symbols.getSymbol("missing", false));
        assertEquals(-1, symbols.getSymbol("missing", "urn:namespace:uri"));
        assertEquals(-1, symbols.getSymbol("missing", "urn:namespace:uri", false));

        assertFalse(symbols.isDirty());
    }

    public void testSimpleSymbol() {
        SymbolTable.SymbolInfo si = symbols.getSymbolInfo((short) 1);
        assertNull(si.getNamespaceURI());
        assertEquals("simple", si.getQName());
        assertEquals(1,        si.getSymbolID());

        assertEquals(1,        symbols.getSymbol("simple"));

        assertFalse(symbols.isDirty());
    }

    public void testNamespacedSymbol() {
        SymbolTable.SymbolInfo si;

        si = symbols.getSymbolInfo((short) 2);
        assertEquals("http://www.w3.org/1999/xhtml", si.getNamespaceURI());
        assertEquals("spaced",                       si.getQName());
        assertEquals(2,                              si.getSymbolID());

        si = symbols.getSymbolInfo((short) 3);
        assertEquals("http://www.w3.org/1999/xhtml", si.getNamespaceURI());
        assertEquals("a:spaced",                     si.getQName());
        assertEquals(3,                              si.getSymbolID());

        si = symbols.getSymbolInfo((short) 4);
        assertEquals("http://www.w3.org/1999/xhtml", si.getNamespaceURI());
        assertEquals("b:spaced",                     si.getQName());
        assertEquals(4,                              si.getSymbolID());

        assertEquals(2,                              symbols.getSymbol("spaced",   "http://www.w3.org/1999/xhtml"));
        assertEquals(3,                              symbols.getSymbol("a:spaced", "http://www.w3.org/1999/xhtml"));
        assertEquals(4,                              symbols.getSymbol("b:spaced", "http://www.w3.org/1999/xhtml"));

        assertEquals(-1,                             symbols.getSymbol("spaced"));
        assertEquals(-1,                             symbols.getSymbol("a:spaced"));
        assertEquals(-1,                             symbols.getSymbol("x:spaced", "http://www.w3.org/1999/xhtml"));

        assertFalse(symbols.isDirty());
    }

    public void testAddSymbol() throws Exception {
        assertEquals(-1, symbols.getSymbol("missing"));
        assertTrue(symbols.getSymbol("missing", true) != -1);
        assertTrue(symbols.isDirty());

        short s = symbols.getSymbol("missing");
        assertTrue(s != -1);

        SymbolTable.SymbolInfo si = symbols.getSymbolInfo(s);
        assertEquals("missing", si.getQName());
        assertNull(si.getNamespaceURI());
        assertEquals(s, si.getSymbolID());
    }

    public void testAddNamespacedSymbol() throws Exception {
        assertEquals(-1, symbols.getSymbol("missing", "urn:unknown"));
        assertTrue(symbols.getSymbol("missing", "urn:unknown", true) != -1);
        assertTrue(symbols.isDirty());

        short s = symbols.getSymbol("missing", "urn:unknown");
        assertTrue(s != -1);

        SymbolTable.SymbolInfo si = symbols.getSymbolInfo(s);
        assertEquals("missing", si.getQName());
        assertEquals("urn:unknown", si.getNamespaceURI());
        assertEquals(s, si.getSymbolID());
    }

    public void testGetNormalizedSymbol() throws Exception {
        assertEquals(1, symbols.getNormalizedSymbol("simple", (NamespaceMap) null, false));

        assertEquals(-1, symbols.getNormalizedSymbol("[http://www.w3.org/1999/xhtml]spaced", (NamespaceMap) null, false));
        assertEquals(-1, symbols.getNormalizedSymbol("[http://www.w3.org/1999/xhtml]a:spaced", (NamespaceMap) null, false));
        assertEquals(-1, symbols.getNormalizedSymbol("[http://www.w3.org/1999/xhtml]b:spaced", (NamespaceMap) null, false));

        short s = symbols.getNormalizedSymbol("[http://www.w3.org/1999/xhtml]spaced", (NamespaceMap) null, true);

        assertEquals(s, symbols.getNormalizedSymbol("[http://www.w3.org/1999/xhtml]spaced", (NamespaceMap) null, false));
        assertEquals(s, symbols.getNormalizedSymbol("[http://www.w3.org/1999/xhtml]a:spaced", (NamespaceMap) null, false));
        assertEquals(s, symbols.getNormalizedSymbol("[http://www.w3.org/1999/xhtml]b:spaced", (NamespaceMap) null, false));
        assertEquals(s, symbols.getNormalizedSymbol("[http://www.w3.org/1999/xhtml]x:spaced", (NamespaceMap) null, false));

        NamespaceMap m = new NamespaceMap();
        m.setNamespace("z", "http://www.w3.org/1999/xhtml");
        assertEquals(s, symbols.getNormalizedSymbol("z:spaced", m, false));
    }

    public void testReadOnly() throws Exception {
        symbols = new SymbolTable(DOMParser.toDocument(SYMBOLS).getDocumentElement(), true);

        assertEquals(-1, symbols.getSymbol("foo"));

        try {
            symbols.getSymbol("foo", true);
            fail();
        } catch (ReadOnlyException e) { }
    }
}
